"""Module providing custom configuration for loguru"""
import json
import sys
from typing import Union

from loguru import logger


def serialize(record):
    """Create custom serializer for logging"""
    exception = record["exception"]

    if exception:
        exception = {
            "type": None if exception.type is None else exception.type.__name__,
            "value": exception.value,
            "traceback": bool(exception.traceback),
        }

    subset = {
        "level": record["level"].name,
        "time": {"repr": record["time"], "timestamp": record["time"].timestamp()},
        "message": record["message"],
        "file": {"name": record["file"].name, "path": record["file"].path},
        "line": record["line"],
        "exception": exception,
        "extra": record["extra"]
    }
    return json.dumps(subset, default=str, ensure_ascii=False) + "\n"


def patching(record):
    """Custom patching for logger serializer"""
    record["extra"]["serialized"] = serialize(record)


def configure_logger(level: Union[str, int]) -> logger:
    """
    Configuration for custom loguru

    :param level: logging level. can be str or int.
    https://docs.python.org/3/library/logging.html#logging-levels
    """

    # Remove default option from loguru, if we don't remove this, it will result in duplicated logs.
    # The pre-configured handler is guaranteed to have the index 0.
    # If there is no active handler with such id, it will throw ValueError. If that happens, continue to create logger.
    try:
        logger.remove(0)
    except ValueError:
        pass

    loguru_logger = logger.patch(patching)  # use custom serializer
    loguru_logger.add(
        sink=sys.stdout,
        level=level,
        format="{extra[serialized]}",
    )

    return loguru_logger



"""Module providing list of function related to date"""
from datetime import datetime
from typing import Union

import pytz


def to_rfc3339(date: datetime) -> str:
    """Return the time formatted according to ISO."""
    return date.isoformat(timespec="microseconds") + "Z"


def to_utc7(date: datetime) -> datetime:
    """Return the time in UTC+7"""
    # Datetime without timezone information will be treated as UTC.
    if not date.tzinfo:
        date = date.replace(tzinfo=pytz.UTC)

    utc7_timezone = pytz.timezone("Asia/Jakarta")

    return date.astimezone(tz=utc7_timezone)


def to_timestamp_millis(dt: Union[datetime, str]) -> int:
    """Converting datetime to unix timestamp without changing timezone.

    Args:
        dt: datetime object or str that will be converted.

    Returns:
        Timestamp (millisecond)
    """
    if isinstance(dt, str):
        dt = datetime.fromisoformat(dt)
    if isinstance(dt, datetime):
        if dt.tzinfo is not None and dt.tzinfo.utcoffset(dt) is not None:
            # remove timezone awareness from timestamptz column value
            dt = dt.replace(tzinfo=None)

    epoch = datetime.utcfromtimestamp(0)
    return (dt - epoch).total_seconds() * 1000


def to_timestamp_without_timezone_literal(date: datetime) -> str:
    """Return the time in %Y-%m-%d %H:%M:%S format"""
    return date.strftime("%Y-%m-%d %H:%M:%S")


def from_datetime_literal(datetime_literal: str, datetime_format: str) -> datetime:
    """Parse string datetime to datetime object.
    Datetime without timezone information will be treated as UTC.

    Args:
        datetime_literal: datetime in string format.
        datetime_format: datetime format.

    Returns:
        datetime object.
    """
    parsed_datetime = datetime.strptime(datetime_literal, datetime_format)
    if not parsed_datetime.tzinfo:
        parsed_datetime = parsed_datetime.replace(tzinfo=pytz.UTC)

    return parsed_datetime
